import re

from django.http import HttpResponse
from django.template.loader import render_to_string
from rest_framework.renderers import BaseRenderer
from xhtml2pdf import pisa

from drf_pdf_renderer.utils import link_callback


class MountPdfMixin:

    def get_template(self):
        if hasattr(self.context['view'], 'pdf_get_renderer_template'):
            return self.context['view'].pdf_get_renderer_template(self.data)
        elif hasattr(self.context['view'], 'pdf_renderer_template'):
            return self.context['view'].pdf_renderer_template
        return 'pdf/list_portrait.html'

    def mount_show_fields(self):
        if hasattr(self.context['view'], 'pdf_get_display_fields'):
            return self.context['view'].pdf_get_display_fields(self.data)
        elif hasattr(self.context['view'], 'pdf_display_fields'):
            return self.context['view'].pdf_display_fields

        return []

    def get_title(self):
        if hasattr(self.context['view'], 'pdf_get_title'):
            return self.context['view'].pdf_get_title(self.data)
        elif hasattr(self.context['view'], 'pdf_title'):
            return self.context['view'].pdf_title

        return re.sub(r"(\w)([A-Z])", r"\1 \2", self.context['view'].__class__.__name__)

    def mount_context(self):
        base_context = {
            'data': self.data, 
            'request': self.context['request'],
            **self.get_additional_context()
        }

        if hasattr(self.context['view'], 'pdf_mount_context'):
            return {
                **self.context['view'].pdf_mount_context(self.data),
                **base_context
            }

        return base_context

    def get_additional_context(self):
        return {
            'fields': self.mount_show_fields(),
            'title': self.get_title()
        }

    def render_template(self):
        return render_to_string(template_name=self.get_template(), context=self.mount_context())

    @staticmethod
    def render_pdf(template):
        return pisa.CreatePDF(template, link_callback=link_callback).dest


class PDFRenderer(MountPdfMixin, BaseRenderer):
    media_type = 'application/pdf'
    format = 'pdf'

    def get_pdf_filename(self, pdf):
        if hasattr(self.context['view'], 'pdf_get_filename'):
            return self.context['view'].pdf_get_filename(pdf, self.data)
        elif hasattr(self.context['view'], 'pdf_filename'):
            return self.context['view'].pdf_filename

        return 'report.pdf'

    def mount_response(self, pdf):
        filename = self.get_pdf_filename(pdf)
        response = HttpResponse(pdf.getvalue(), content_type='application/pdf')
        response['Content-Disposition'] = f'attachment; filename={filename}'
        return response

    def process_render(self):
        if self.data is None or self.context['response'].status_code >= 300:
            return self.data

        template = self.render_template()
        pdf = self.render_pdf(template)
        return self.mount_response(pdf)

    def render(self, data, accepted_media_type=None, renderer_context=None):
        self.data = data
        self.context = renderer_context
        return self.process_render()


class PDFRendererPaginated(PDFRenderer):
    results_field = 'results'

    def render(self, data, *args, **kwargs):
        if not isinstance(data, list):
            data = data.get(self.results_field, [])
        return super().render(data, *args, **kwargs)



from rest_framework.settings import api_settings

from drf_pdf_renderer.renderer import PDFRendererPaginated


class PdfAllResultsMixin:
    renderer_classes = (*api_settings.DEFAULT_RENDERER_CLASSES, PDFRendererPaginated)

    def paginate_queryset(self, queryset):
        if self.paginator and self.request.accepted_renderer.format == "pdf":
            self.paginator.page_size = 99999
        return super().paginate_queryset(queryset)



import os

from django.conf import settings
from django.db.models.fields.related import ForeignKey


def link_callback(uri, rel):
    """
    Convert HTML URIs to absolute system paths so xhtml2pdf can access those
    resources
    """
    base_path = settings.BASE_DIR
    result, exists = f"{base_path}{uri}", os.path.exists(f"{base_path}{uri}")

    if exists:
        if not isinstance(result, (list, tuple)):
            result = [result]
        result = list(os.path.realpath(path) for path in result)
        path = result[0]
    else:
        static_url = settings.STATIC_URL        # Typically /static/
        static_root = settings.STATIC_ROOT      # Typically /home/userX/project_static/
        media_url = settings.MEDIA_URL         # Typically /media/
        media_root = settings.MEDIA_ROOT       # Typically /home/userX/project_static/media/

        if uri.startswith(media_url):
            path = os.path.join(media_root, uri.replace(media_url, ""))
        elif uri.startswith(static_url):
            path = os.path.join(static_root, uri.replace(static_url, ""))
        else:
            return uri

    # make sure that file exists
    if not os.path.isfile(path):
        raise Exception('media URI must start with %s or %s' % (static_url, media_url))
    return path



from datetime import datetime

from django import template
from django.template.defaultfilters import date as date_filter


register = template.Library()


@register.filter
def isodate(value, arg):
    return date_filter(datetime.strptime(value, "%Y-%m-%d").date(), arg) if value else ''


@register.filter
def format_cpf(value):
    return "{}{}{}.{}{}{}.{}{}{}-{}{}".format(*value) if len(value) == 11 else value


@register.filter(name='lookup')
def lookup(dictionary, key):
    return dictionary.get(key)



# -*- coding: utf-8 -*-
try:
    import django
except ImportError:
    django = None

__version__ = '0.1.2'

if django and django.VERSION < (3, 2):  # pragma: no cover
    default_app_config = 'drf_pdf_renderer.apps.DjangoPdfRendererConfig'



# -*- coding: utf-8 -*-
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoPdfRendererConfig(AppConfig):  # Our app config class
    name = 'drf_pdf_renderer'
    verbose_name = _('Django PDF Renderer')



# -*- coding: utf-8 -*-
#!/usr/bin/env python
from io import open

from setuptools import find_packages, setup

from drf_pdf_renderer import __version__

setup(
    name='drf-pdf-renderer',
    version=__version__,
    description='A simplistic/very extendable pdf renderer.',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='Shinneider Libanio da Silva',
    author_email='shinneider-libanio@hotmail.com',
    url='https://github.com/shinneider/drf-pdf-renderer',
    license='MIT',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    python_requires=">=3.3",
    install_requires=[
        'django',
        'djangorestframework'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
